@@ -16,6 +16,17 @@ def test_class_train(data_dir, image_names):
     print('>>>> model trained and saved to %s'%cpmodel_path)
         
 def test_cli_train(data_dir, image_names):
+    train_dir = str(data_dir.joinpath('2D').joinpath('train'))
+    model_dir = str(data_dir.joinpath('2D').joinpath('train').joinpath('models'))
+    shutil.rmtree(model_dir, ignore_errors=True)
+    cmd = 'python -m cellpose --train --train_size --n_epochs 10 --dir %s --mask_filter _cyto_masks --pretrained_model None --chan 2 --chan2 1 --diameter 17'%train_dir
+    try:
+        cmd_stdout = check_output(cmd, stderr=STDOUT, shell=True).decode()
+    except Exception as e:
+        print(e) 
+        raise ValueError(e)
+
+def test_cli_train_pretrained(data_dir, image_names):
     train_dir = str(data_dir.joinpath('2D').joinpath('train'))
     model_dir = str(data_dir.joinpath('2D').joinpath('train').joinpath('models'))
     shutil.rmtree(model_dir, ignore_errors=True)
